@@ -28,6 +28,7 @@ from typecatcher_lib import Window
 from typecatcher.AboutTypeCatcherDialog import AboutTypeCatcherDialog
 from typecatcher.FindFonts import FindFonts
 from typecatcher_lib.xdg import fontDir
+from typecatcher_lib.helpers import running_gnome_shell
 from typecatcher.DownloadFont import DownloadFont, UninstallFont, DownloadError
 from typecatcher.html_preview import html_font_view, select_text_preview
 
@@ -50,6 +51,7 @@ class TypeCatcherWindow(Window):
         context = self.toolbar.get_style_context()
         context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
         self.view = WebKit.WebView()
+        self.view.connect('console-message', self.on_js_console_message)
         webview = builder.get_object("webview")
         webview.add(self.view)
         htmlfile = html_font_view()
@@ -57,7 +59,8 @@ class TypeCatcherWindow(Window):
         webview.show_all()
 
         webview_settings = self.view.get_settings()
-        webview_settings.set_property('enable-default-context-menu', False)
+        webview_settings.set_property('enable-default-context-menu',
+                                      False)
 
         # the data in the model
         listmodel = builder.get_object("liststore")
@@ -84,9 +87,45 @@ class TypeCatcherWindow(Window):
         self.mnu_save = self.builder.get_object('mnu_save')
         self.mnu_save.connect("activate", self.on_download_btn_clicked)
 
-        self.text_selector = self.builder.get_object('text_selector')
         self.text_menu = self.builder.get_object('text_menu')
-        self.text_selector.set_menu(self.text_menu)
+
+        # Fallback to known icon for backwards compat
+        icon_theme = Gtk.IconTheme.get_default()
+        if icon_theme.has_icon('view-more-symbolic'):
+            menu_icon = 'view-more-symbolic'
+        else:
+            logger.debug("Falling back to go-down-symbolic")
+            menu_icon = 'go-down-symbolic'
+
+        try: # Try to use a GtkMenuButton, fallback to a ToolButton
+            self.selector_icon = Gtk.Image.new()
+            self.selector_icon.set_from_icon_name(menu_icon,
+                                                  Gtk.IconSize.LARGE_TOOLBAR)
+
+            self.text_selector = Gtk.MenuButton.new()
+            self.text_selector.set_popup(self.text_menu)
+            self.text_selector.add(self.selector_icon)
+
+            self.selector_tool_item = Gtk.ToolItem.new()
+            self.selector_tool_item.add(self.text_selector)
+            self.selector_tool_item.set_margin_left(4)
+
+            self.text_selector.show()
+            self.selector_tool_item.show()
+            self.selector_icon.show()
+            self.toolbar.insert(self.selector_tool_item, 6)
+
+        except AttributeError: # Gtk version is too old.
+            logger.debug("Falling back to GtkToolButton")
+            self.text_selector = Gtk.ToolButton.new(None, None)
+            self.text_selector.set_icon_name(menu_icon)
+            self.text_selector.set_homogeneous(False)
+            self.text_selector.set_expand(False)
+            self.toolbar.insert(self.text_selector, 6)
+            self.text_selector.connect('clicked', self.on_text_selector_clicked)
+            self.text_selector.show()
+
+        self.text_selector.set_tooltip_text(_("Select text..."))
 
         radios = ['r1', 'r2', 'r3', 'r4', 'r5', 'r6', 'r7', 'r8']
         for position, item in enumerate(radios):
@@ -100,6 +139,11 @@ class TypeCatcherWindow(Window):
         self.scale.set_value(36)
         self.scale.connect("value-changed", self.spin_moved)
 
+        self.menubar = self.builder.get_object('menubar')
+        # Hide menubar under GNOME
+        if running_gnome_shell():
+            self.menubar.hide()
+
     def spin_moved(self, widget):
         size = int(self.scale.get_value())
         text_size = "document.getElementById('text_preview').style.fontSize = '%s';" % size
@@ -134,25 +178,16 @@ class TypeCatcherWindow(Window):
                    """
                    document.getElementById('text_preview').style.fontFamily = "\'%s\'";
                    """  % (self.font)]
-        font_loader = """WebFontConfig = {
-        google: { families: [ '%s' ] },
-        inactive: function() {
-          document.getElementById('start_page').style.display = 'None';
-          document.getElementById('text_preview').style.display = 'None';
-          document.getElementById('no_connect').style.display = 'block';
-        },
-      }; 
-      (function() {
-        document.getElementsByTagName("html")[0].setAttribute("class","wf-loading")
-        document.getElementsByTagName("html")[0].setAttribute("className","wf-loading")
-        var wf = document.createElement('script');
-        wf.src = ('https:' == document.location.protocol ? 'https' : 'http') +
-            '://ajax.googleapis.com/ajax/libs/webfont/1.4.10/webfont.js';
-        wf.type = 'text/javascript';
-        wf.async = 'true';
-        var s = document.getElementsByTagName('script')[0];
-        s.parentNode.insertBefore(wf, s);
-      })();""" % (self.font)
+        font_loader = """WebFont.load({
+            google: { families: [ '%s' ] },
+            fontinactive: function(font, fvd) {
+                // 
+                console.log("Couldn't load " + font + " FVD:" + fvd);
+                document.getElementById('start_page').style.display = 'None';
+                document.getElementById('text_preview').style.display = 'None';
+                document.getElementById('no_connect').style.display = 'block';
+            },
+        });""" % (self.font)
         show_text = [font_loader,
                      "document.getElementById('no_connect').style.display = 'None';",
                      "document.getElementById('text_preview').style.display = 'block';"]
@@ -182,7 +217,7 @@ class TypeCatcherWindow(Window):
         fonts = list(itertools.chain(*self.fonts))
         entered_text = self.search_field.get_text()
         matcher = re.compile(entered_text, re.IGNORECASE)
-        if any(itertools.ifilter(matcher.match, fonts)):
+        if any(filter(matcher.match, fonts)):
             for position, item in enumerate(fonts):
                 if item.lower() == entered_text.lower():
                     self.listview.set_cursor(position)
@@ -190,8 +225,8 @@ class TypeCatcherWindow(Window):
             pass
 
     def on_match_selected(self, completion, treemodel, treeiter):
-        self.font = treemodel[treeiter][completion.get_text_column()]
-        self.js_exec()
+        path = treemodel.get_path(treeiter)
+        self.listview.set_cursor(path)
 
     def on_select_changed(self, selection):
         (model, iter) = selection.get_selected()
@@ -251,3 +286,12 @@ class TypeCatcherWindow(Window):
             dialog.destroy()
         except AttributeError:
             pass
+
+    def on_text_selector_clicked(self, widget):
+        self.text_menu.popup(None, None, None, None, 0, Gtk.get_current_event_time())
+
+    def on_js_console_message(self, webview, message, line, id, data=None):
+        """
+        Log messages to JavaScript console
+        """
+        logger.debug(message)
